@@ -72,14 +72,17 @@ class Schedule(PushEventSource):
     principal = 'events.amazonaws.com'
     property_types = {
             'Schedule': PropertyType(True, is_str()),
-            'Input': PropertyType(False, is_str())
+            'Input': PropertyType(False, is_str()),
+            'Enabled': PropertyType(False, is_type(bool)),
+            'Name': PropertyType(False, is_str()),
+            'Description': PropertyType(False, is_str())
     }
 
     def to_cloudformation(self, **kwargs):
         """Returns the CloudWatch Events Rule and Lambda Permission to which this Schedule event source corresponds.
 
         :param dict kwargs: no existing resources need to be modified
-        :returns: a list of vanilla CloudFormation Resources, to which this pull event expands
+        :returns: a list of vanilla CloudFormation Resources, to which this Schedule event expands
         :rtype: list
         """
         function = kwargs.get('function')
@@ -93,6 +96,10 @@ class Schedule(PushEventSource):
         resources.append(events_rule)
 
         events_rule.ScheduleExpression = self.Schedule
+        if self.Enabled is not None:
+            events_rule.State = "ENABLED" if self.Enabled else "DISABLED"
+        events_rule.Name = self.Name
+        events_rule.Description = self.Description
         events_rule.Targets = [self._construct_target(function)]
 
         source_arn = events_rule.get_runtime_attr("arn")
@@ -123,9 +130,9 @@ class CloudWatchEvent(PushEventSource):
     resource_type = 'CloudWatchEvent'
     principal = 'events.amazonaws.com'
     property_types = {
-            'Pattern': PropertyType(False, is_type(dict)),
-            'Input': PropertyType(False, is_str()),
-            'InputPath': PropertyType(False, is_str())
+        'Pattern': PropertyType(False, is_type(dict)),
+        'Input': PropertyType(False, is_str()),
+        'InputPath': PropertyType(False, is_str())
     }
 
     def to_cloudformation(self, **kwargs):
@@ -133,7 +140,7 @@ class CloudWatchEvent(PushEventSource):
         corresponds.
 
         :param dict kwargs: no existing resources need to be modified
-        :returns: a list of vanilla CloudFormation Resources, to which this pull event expands
+        :returns: a list of vanilla CloudFormation Resources, to which this CloudWatch Events event expands
         :rtype: list
         """
         function = kwargs.get('function')
@@ -344,6 +351,7 @@ class SNS(PushEventSource):
     principal = 'sns.amazonaws.com'
     property_types = {
             'Topic': PropertyType(True, is_str()),
+            'Region': PropertyType(False, is_str()),
             'FilterPolicy': PropertyType(False, dict_of(is_str(), list_of(one_of(is_str(), is_type(dict)))))
     }
 
@@ -360,13 +368,15 @@ class SNS(PushEventSource):
             raise TypeError("Missing required keyword argument: function")
 
         return [self._construct_permission(function, source_arn=self.Topic),
-                self._inject_subscription(function, self.Topic, self.FilterPolicy)]
+                self._inject_subscription(function, self.Topic, self.Region, self.FilterPolicy)]
 
-    def _inject_subscription(self, function, topic, filterPolicy):
+    def _inject_subscription(self, function, topic, region, filterPolicy):
         subscription = SNSSubscription(self.logical_id)
         subscription.Protocol = 'lambda'
         subscription.Endpoint = function.get_runtime_attr("arn")
         subscription.TopicArn = topic
+        if region is not None:
+            subscription.Region = region
         if CONDITION in function.resource_attributes:
             subscription.set_resource_attribute(CONDITION, function.resource_attributes[CONDITION])
 
@@ -492,7 +502,7 @@ class Api(PushEventSource):
         if not stage or not suffix:
             raise RuntimeError("Could not add permission to lambda function.")
 
-        path = path.replace('{proxy+}', '*')
+        path = re.sub(r'{([a-zA-Z0-9._-]+|proxy\+)}', '*', path)
         method = '*' if self.Method.lower() == 'any' else self.Method.upper()
 
         api_id = self.RestApiId
@@ -536,9 +546,9 @@ class Api(PushEventSource):
 
         if self.Auth:
             method_authorizer = self.Auth.get('Authorizer')
+            api_auth = api.get('Auth')
 
             if method_authorizer:
-                api_auth = api.get('Auth')
                 api_authorizers = api_auth and api_auth.get('Authorizers')
 
                 if method_authorizer != 'AWS_IAM':
@@ -563,6 +573,16 @@ class Api(PushEventSource):
                             'is only a valid value when a DefaultAuthorizer on the API is specified.'.format(
                                 method=self.Method, path=self.Path))
 
+            apikey_required_setting = self.Auth.get('ApiKeyRequired')
+            apikey_required_setting_is_false = apikey_required_setting is not None and not apikey_required_setting
+            if apikey_required_setting_is_false and not api_auth.get('ApiKeyRequired'):
+                raise InvalidEventException(
+                    self.relative_id,
+                    'Unable to set ApiKeyRequired [False] on API method [{method}] for path [{path}] '
+                    'because the related API does not specify any ApiKeyRequired.'.format(
+                        method=self.Method, path=self.Path))
+
+            if method_authorizer or apikey_required_setting is not None:
                 editor.add_auth_to_method(api=api, path=self.Path, method_name=self.Method, auth=self.Auth)
 
         if self.RequestModel:
