@@ -1,13 +1,17 @@
 import boto3
+import json
 from botocore.exceptions import ClientError, EndpointConnectionError
 import logging
 from time import sleep, time
+import copy
 
 from samtranslator.model.exceptions import InvalidResourceException
 from samtranslator.plugins import BasePlugin
 from samtranslator.plugins.exceptions import InvalidPluginException
 from samtranslator.public.sdk.resource import SamResourceType
 from samtranslator.public.sdk.template import SamTemplate
+from samtranslator.intrinsics.resolver import IntrinsicsResolver
+from samtranslator.intrinsics.actions import FindInMapAction
 
 
 class ServerlessAppPlugin(BasePlugin):
@@ -35,7 +39,7 @@ class ServerlessAppPlugin(BasePlugin):
     LOCATION_KEY = 'Location'
     TEMPLATE_URL_KEY = 'TemplateUrl'
 
-    def __init__(self, sar_client=None, wait_for_template_active_status=False, validate_only=False):
+    def __init__(self, sar_client=None, wait_for_template_active_status=False, validate_only=False, parameters={}):
         """
         Initialize the plugin.
 
@@ -50,6 +54,7 @@ class ServerlessAppPlugin(BasePlugin):
         self._sar_client = sar_client
         self._wait_for_template_active_status = wait_for_template_active_status
         self._validate_only = validate_only
+        self._parameters = parameters
 
         # make sure the flag combination makes sense
         if self._validate_only is True and self._wait_for_template_active_status is True:
@@ -68,6 +73,7 @@ class ServerlessAppPlugin(BasePlugin):
         :return: Nothing
         """
         template = SamTemplate(template_dict)
+        intrinsic_resolvers = self._get_intrinsic_resolvers(template_dict.get('Mappings', {}))
 
         service_call = None
         if self._validate_only:
@@ -79,9 +85,19 @@ class ServerlessAppPlugin(BasePlugin):
                 # Handle these cases in the on_before_transform_resource event
                 continue
 
-            app_id = app.properties[self.LOCATION_KEY].get(self.APPLICATION_ID_KEY)
-            semver = app.properties[self.LOCATION_KEY].get(self.SEMANTIC_VERSION_KEY)
+            app_id = self._replace_value(app.properties[self.LOCATION_KEY],
+                                         self.APPLICATION_ID_KEY, intrinsic_resolvers)
+
+            semver = self._replace_value(app.properties[self.LOCATION_KEY],
+                                         self.SEMANTIC_VERSION_KEY, intrinsic_resolvers)
+
+            if isinstance(app_id, dict) or isinstance(semver, dict):
+                key = (json.dumps(app_id), json.dumps(semver))
+                self._applications[key] = False
+                continue
+
             key = (app_id, semver)
+
             if key not in self._applications:
                 try:
                     # Lazy initialization of the client- create it when it is needed
@@ -92,6 +108,21 @@ class ServerlessAppPlugin(BasePlugin):
                     # Catch all InvalidResourceExceptions, raise those in the before_resource_transform target.
                     self._applications[key] = e
 
+    def _replace_value(self, input_dict, key, intrinsic_resolvers):
+        value = self._resolve_location_value(input_dict.get(key), intrinsic_resolvers)
+        input_dict[key] = value
+        return value
+
+    def _get_intrinsic_resolvers(self, mappings):
+        return [IntrinsicsResolver(self._parameters),
+                IntrinsicsResolver(mappings, {FindInMapAction.intrinsic_name: FindInMapAction()})]
+
+    def _resolve_location_value(self, value, intrinsic_resolvers):
+        resolved_value = copy.deepcopy(value)
+        for intrinsic_resolver in intrinsic_resolvers:
+            resolved_value = intrinsic_resolver.resolve_parameter_refs(resolved_value)
+        return resolved_value
+
     def _can_process_application(self, app):
         """
         Determines whether or not the on_before_transform_template event can process this application
@@ -101,6 +132,7 @@ class ServerlessAppPlugin(BasePlugin):
         return (self.LOCATION_KEY in app.properties and
                 isinstance(app.properties[self.LOCATION_KEY], dict) and
                 self.APPLICATION_ID_KEY in app.properties[self.LOCATION_KEY] and
+                app.properties[self.LOCATION_KEY][self.APPLICATION_ID_KEY] is not None and
                 self.SEMANTIC_VERSION_KEY in app.properties[self.LOCATION_KEY])
 
     def _handle_get_application_request(self, app_id, semver, key, logical_id):
@@ -188,11 +220,23 @@ class ServerlessAppPlugin(BasePlugin):
                                        [self.APPLICATION_ID_KEY, self.SEMANTIC_VERSION_KEY])
 
         app_id = resource_properties[self.LOCATION_KEY].get(self.APPLICATION_ID_KEY)
+
         if not app_id:
             raise InvalidResourceException(logical_id, "Property 'ApplicationId' cannot be blank.")
+
+        if isinstance(app_id, dict):
+            raise InvalidResourceException(logical_id, "Property 'ApplicationId' cannot be resolved. Only FindInMap "
+                                                       "and Ref intrinsic functions are supported.")
+
         semver = resource_properties[self.LOCATION_KEY].get(self.SEMANTIC_VERSION_KEY)
+
         if not semver:
-            raise InvalidResourceException(logical_id, "Property 'SemanticVersion cannot be blank.")
+            raise InvalidResourceException(logical_id, "Property 'SemanticVersion' cannot be blank.")
+
+        if isinstance(semver, dict):
+            raise InvalidResourceException(logical_id, "Property 'SemanticVersion' cannot be resolved. Only FindInMap "
+                                                       "and Ref intrinsic functions are supported.")
+
         key = (app_id, semver)
 
         # Throw any resource exceptions saved from the before_transform_template event
