@@ -44,7 +44,7 @@ class Globals(object):
         #   StageName: Because StageName cannot be overridden for Implicit APIs because of the current plugin
         #              architecture
         SamResourceType.Api.value: [
-            'Auth',
+            "Auth",
             "Name",
             "DefinitionUri",
             "CacheClusterEnabled",
